
# q-expansion of newform 722.2.c.b, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 722
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[363, [0, -1]]]
aps_data = [[-1, 1], [1, -1], [0, 0], [-4, 0], [3, 0], [0, 2], [6, -6], [0, 0], [0, 6], [0, 0], [-2, 0], [10, 0], [9, -9], [4, -4], [0, 0], [0, 6], [-9, 9], [0, 4], [0, -7], [-6, 6], [1, -1], [-4, 4], [3, 0], [0, 6], [17, -17], [0, 0], [-2, 0], [0, 0], [-16, 16], [-15, 0], [0, 2], [-9, 9], [0, -9], [0, -11], [-18, 18], [10, 0], [16, -16], [-19, 0], [0, -24], [6, -6], [-9, 0], [0, 2], [12, 0], [2, -2], [18, 0], [0, 10], [20, -20], [14, -14], [-3, 0], [-16, 0], [-3, 3], [-12, 0], [0, 5], [0, 3], [0, 3], [12, -12], [12, -12], [16, -16], [8, 0], [0, 27], [-5, 5], [-24, 0], [-7, 7], [-30, 0], [0, 19], [0, -18], [-5, 0], [11, -11], [9, -9], [-4, 0], [3, 0], [6, -6], [0, 22], [4, 0], [28, 0], [-36, 36], [0, 36], [10, -10], [-27, 27], [0, 5], [-12, 0], [-10, 10], [0, -30], [0, 26], [14, -14], [0, -9], [-9, 0], [5, 0], [-6, 6], [-34, 0], [-27, 0], [0, -36], [-2, 0], [0, 0], [25, -25], [0, -6], [0, -24], [-9, 0], [0, -28], [0, -44], [0, -4], [0, -24], [21, 0], [6, 0], [-7, 0], [11, 0], [12, -12], [0, 21], [0, -6], [13, 0], [-20, 0], [-2, 2], [0, -3], [-4, 0], [0, 28], [-39, 39], [43, -43], [18, 0], [-12, 0], [0, 36], [0, -40], [-14, 0], [42, 0], [36, 0], [44, 0], [24, -24], [0, -14], [-30, 30], [-32, 32], [-22, 0], [-35, 35], [-18, 18], [0, 38], [10, -10], [-39, 0], [0, -2], [0, -48], [7, 0], [-6, 0], [45, 0], [0, -16], [0, -18], [0, -14], [0, -39], [-44, 0], [-12, 12], [22, -22], [-3, 3], [0, 43], [-18, 0], [20, -20], [-9, 0], [0, 19], [0, 48], [17, -17], [-30, 0], [-10, 0], [3, -3], [0, -35], [0, -42], [-60, 60], [-15, 15], [34, -34], [21, -21], [-33, 0], [24, -24], [8, -8], [0, 4]]
